import json
from typing import Optional

import aiohttp
from v6d0auth import certs
from v6d0auth.client import mycert, with_role

from v6d1tokens.config import taurl

__all__ = ('request_token',)


async def request_token(query: str, query_mode: Optional[str] = None, tail: str = '') -> str:
    async with aiohttp.ClientSession() as session:
        if query_mode is None:
            if tail:
                raise ValueError('tail is used only for non-None query modes')
            request = session.post(
                f'{taurl}/get',
                data=json.dumps([query, (await mycert()).hex()]).encode()
            )
        else:
            if query_mode == 'token':
                token_id, role = query, f'token:{query}::{tail}'
            elif query_mode == 'role':
                token_id, role = f'role:{query}::{tail}', query
            else:
                raise ValueError('unknown query mode')
            await with_role(role)
            request = session.post(
                f'{taurl}/get',
                data=json.dumps([token_id, certs.vkey.encode().hex()]).encode(),
                headers={'v6role': role}
            )
        async with request as response:
            if response.status == 200:
                return certs.receive(await response.read()).decode()
            else:
                raise RuntimeError(response.status)
